local enginetable = {
	[1] = {{1,3},{1,3}},
	[2] = {{4,6},{1,3}},
	[3] = {{7,9},{1,3}},
	[4] = {{1,3},{4,6}},
	[5] = {{4,6},{4,6}},
	[6] = {{7,9},{4,6}},
	[7] = {{1,3},{7,9}},
	[8] = {{4,6},{7,9}},
	[9] = {{7,9},{7,9}}
}

local boostsounds = {
	sfx_fzabst,
	sfx_fzbbst,
	sfx_fzcbst,
	sfx_fzdbst,
	sfx_fzebst,
	sfx_fzfbst,
	sfx_fzgbst,
	sfx_fzhbst,
	sfx_fzibst
}

local lapcount = {
	[2] = sfx_fzl2,
	[3] = sfx_fzl3,
	[4] = sfx_fzl4,
	[5] = sfx_fzl5,
	[6] = sfx_fzl6,
	[7] = sfx_fzl7,
	[8] = sfx_fzl8,
	[9] = sfx_fzl9,
	[10] = sfx_fzl10,
	[15] = sfx_fzl15,
	[20] = sfx_fzl20,
	[30] = sfx_fzl30,
	[40] = sfx_fzl40,
	[50] = sfx_fzl50,
	[60] = sfx_fzl60
}
FZeroKart.GetEngineClass = function(p)
	for i = 1, #enginetable
		if (p.kartspeed >= enginetable[i][1][1] and p.kartspeed <= enginetable[i][1][2])
		and (p.kartweight >= enginetable[i][2][1] and p.kartweight <= enginetable[i][2][2]) then
			--print("Engine Class: "..i)
			return i
		end
	end
	
	return 9 -- Fallback if lua puts stats outside standard engine class
end

local finish = {
	[1] = {
			sfx_fzfn1,
			sfx_fz1st2,
			sfx_fzal1
	},
	
	[2] = {
			sfx_fzfn2,
			sfx_fzal2
	},
	
	[3] = {
			sfx_fzfn3,
			sfx_fzal3
	},

	[4] = {
			sfx_fzfn4,
			sfx_fzal4
	}
}
local finishtable = {
	sfx_fzfn5,
	sfx_fzfn6,
	sfx_fzfn7,
	sfx_fzfn8,
	sfx_fzfn9,
	sfx_fzfn10,
	sfx_fzfn11,
	sfx_fzfn12,
	sfx_fzfn13,
	sfx_fzfn14,
	sfx_fzfn15,
	sfx_fzfn16
}


local function GXFinish(p)
	local truepos
	local timetable = {}
	for t in players.iterate
		if t.spectator then continue end
		if (t.exiting > 0 and not(t.pflags & PF_TIMEOVER)) and (t.mo and t.mo.valid) then
			table.insert(timetable, t)
		end
	end
	table.sort(timetable, function(a,b)
		return a.realtime < b.realtime
	end)
	for i = 1, #timetable
		local tietracker = timetable[i-1] -- potential tie
		if timetable[i] == #p then
			if (tietracker and timetable[i].realtime == tietracker.realtime) then
				p.kartstuff[k_position] = tietracker.kartstuff[k_position]
				break
			else
				p.kartstuff[k_position] = i
				break
			end
		end
	end
				
	local finishref
	local pos = p.kartstuff[k_position]
				
	if pos <= 4 then
		finishref = finish[pos]
	end

	p.fzerokart.finishsound = pos <= 4 and finishref[N_RandomRange(1, #finishref)] or finishtable[(pos-4)]
	S_StartSound(p.mo, p.fzerokart.finishsound, p)
	p.fzerokart.raceeval = true
end

FZeroKart.PostThink = function()
	if not FZeroKart.Active.value then return end
	
	if leveltime == 130 then // Countdown
		S_StartSound(nil, sfx_fzc3)
	end
	
	if leveltime == 165 then
		S_StartSound(nil, sfx_fzc2)
	end
	if leveltime == 200 then
		S_StartSound(nil, sfx_fzc1)
	end
	if leveltime == 235 then
		S_StartSound(nil, sfx_fzgo)
	end

	for p in players.iterate
		if p.spectator then continue end
		if not p.fzerokart then continue end
		p.laps = $ or 0
		p.fzerokart.lastlap = $ or 0
		
		if p.laps > p.fzerokart.lastlap
		and not p.exiting then
			--print("new lap")
			if p.laps == numlaps - 1 then
				S_StartSound(nil, sfx_fzfl, p)
			else
				local lapsound = lapcount[p.laps+1]
				if lapsound then
					S_StartSound(nil, lapsound, p)
				end
			end
		
		elseif p.exiting then
			if not S_SoundPlaying(p.mo, sfx_fzfini)
			and not p.fzerokart.racefinish then
				S_StartSound(p.mo, sfx_fzfini, p)
				p.fzerokart.racefinish = true
			end
			
			if not S_SoundPlaying(p.mo, sfx_fzfini, p)
			and p.fzerokart.racefinish
			and not p.fzerokart.raceeval then
				-- We want consistent finish audio; fuck jankpoints
				GXFinish(p)
			end
		end
	
	
		if S_SoundPlaying(p.mo, sfx_cdfm01) then
			S_StopSoundByID(p.mo, sfx_cdfm01)
			S_StartSound(p.mo, boostsounds[FZeroKart.GetEngineClass(p)])
		end

		if S_SoundPlaying(p.mo, sfx_s26d) then -- TURN AROUND, DUMBASS
			S_StopSoundByID(p.mo, sfx_s26d)
			S_StartSound(p.mo, sfx_fzwrng)
		end
		
		if p.kartstuff[k_respawn] == 2 then
			S_StartSound(p.mo, sfx_fzrest, p)
		end

		p.fzerokart.lastlap = p.laps
	end
end
addHook("PostThinkFrame", FZeroKart.PostThink)